@@ -1,19 +1,23 @@
-# Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import grt
 import mforms
@@ -24,6 +28,7 @@ from workbench.graphics.canvas import ImageFigure
 from wb_admin_perfschema_instrumentation_be import PSConfiguration, PSTimerType, PSObject, PSActor
 
 from workbench.log import log_info, log_error
+from workbench.notifications import nc
 
 from wb_admin_perfschema import WbAdminPSBaseTab
 
@@ -68,8 +73,8 @@ class BigSwitch(mforms.PyDrawBox):
         self.hovering_state = None
         self.mouse_pos = None, None
 
-        self.text_image = ImageFigure(mforms.App.get().get_resource_path("ps_switcher_text.png"))
-        self.legend = ImageFigure(mforms.App.get().get_resource_path("ps_switcher_legende.png"))
+        self.updateImage()
+        self.legend = ImageFigure(mforms.App.get().get_resource_path("ps_switcher_legend.png"))
 
         self.state_images = []
         for item in ["fully", "custom", "default", "disabled"]:
@@ -106,7 +111,6 @@ class BigSwitch(mforms.PyDrawBox):
         self.legend.render(c)
 
 
-
     def set_state(self, state):
         self.state = state
         self.set_needs_repaint()
@@ -121,6 +125,12 @@ class BigSwitch(mforms.PyDrawBox):
         if self.hovering_state:
             self.callback(self.hovering_state)
 
+    def updateImage(self):
+        if mforms.App.get().isDarkModeActive():
+            self.text_image = ImageFigure(mforms.App.get().get_resource_path("ps_switcher_text_dark.png"))
+        else:
+            self.text_image = ImageFigure(mforms.App.get().get_resource_path("ps_switcher_text_light.png"))
+
 
 class EasySetupPage(mforms.Box):
     def __init__(self, owner):
@@ -128,9 +138,10 @@ class EasySetupPage(mforms.Box):
         self.set_managed()
         self.set_release_on_add()
         self.set_spacing(12)
-        self.set_padding(12)
+        self.set_padding(20)
         self.owner = owner
 
+        nc.add_observer(self.updateColors, "GNColorsChanged")
         self.create_ui()
 
 
@@ -138,13 +149,15 @@ class EasySetupPage(mforms.Box):
         try:
             r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_consumers WHERE enabled='NO'")
             if r == 0:
-                r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_instruments WHERE enabled='NO' OR timed='NO'")
+                # Exclude results from 'memory/%' because they can't be disabled
+                r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_instruments WHERE NAME NOT LIKE 'memory/%' AND (enabled='NO' OR timed='NO')")
                 if r == 0:
                     return "fully"
 
             r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_consumers WHERE enabled='YES'")
             if r == 0:
-                r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_instruments WHERE enabled='YES' OR timed='YES'")
+                # Exclude results from 'memory/%' because they can't be disabled
+                r = self.owner.get_select_int_result("SELECT COUNT(*) FROM performance_schema.setup_instruments WHERE NAME NOT LIKE 'memory/%' AND (enabled='YES' OR timed='YES')")
                 if r == 0:
                     return "disabled"
 
@@ -164,6 +177,8 @@ class EasySetupPage(mforms.Box):
                 nlikes = []
                 likes = []
                 ins = []
+                # Exclude results from 'memory/%' because they can't be disabled
+                nlikes.append("'memory/%'")
                 for i in instruments:
                    if '%' in i:
                        likes.append("NAME LIKE '%s'" % i)
@@ -260,16 +275,21 @@ class EasySetupPage(mforms.Box):
         self.logo.set_image(mforms.App.get().get_resource_path("ps_easysetup_logo.png"))
         self.add(self.logo, False, True)
 
+        label = mforms.newLabel("Performance Schema")
+        label.set_text_align(mforms.MiddleCenter)
+        label.set_style(mforms.VeryBigStyle)
+        self.add(label, False, True)
+
         self.switch_image = BigSwitch()
         self.switch_image.callback = self.change_instrumentation
-        self.switch_image.set_size(200, 152)
+        self.switch_image.set_size(105, 140)
         self.add(self.switch_image, False, True)
 
         image = mforms.newImageBox()
         image.set_image(mforms.App.get().get_resource_path("separator_vertical.png"))
         self.add(image, False, True)
 
-        label = mforms.newLabel("\n\nThe MySQL Performance Schema allows to:\n- instrument MySQL to collect statistics and performance data\n- log collected events into tables, so they can be analyzed\n\nUse the switch above to change Performance Schema instrumentation or disable it.")
+        label = mforms.newLabel("\n\nThe MySQL Performance Schema allows you to:\n- instrument MySQL to collect statistics and performance data\n- log collected events into tables, so they can be analyzed\n\nUse the switch above to change Performance Schema instrumentation or disable it.")
         label.set_text_align(mforms.MiddleCenter)
         self.add(label, False, True)
 
@@ -280,6 +300,8 @@ class EasySetupPage(mforms.Box):
         self.logo.set_image(mforms.App.get().get_resource_path("ps_easysetup_logo_enabled.png" if state != "disabled" else "ps_easysetup_logo.png"))
         self.switch_image.set_state(state)
 
+    def updateColors(self, name, sender, info):
+        self.switch_image.updateImage()
 
 class SetupInstruments(mforms.Box):
     def __init__(self, owner, data):
@@ -308,9 +330,9 @@ The enabled column indicates whether the events for the instrument are generated
 The timed column indicates whether information about the event duration is recorded.
 """
         label = mforms.newLabel(description)
-        self.add(label, False, False)
+        self.add(label, False, True)
         
-        self._instruments = mforms.newTreeNodeView(mforms.TreeAltRowColors)
+        self._instruments = mforms.newTreeView(mforms.TreeAltRowColors)
         self._instruments.add_column(mforms.IconStringColumnType, "Instrument", 400, False)
         self._instruments.add_column(mforms.TriCheckColumnType, "Enabled", 50, True)
         self._instruments.add_column(mforms.TriCheckColumnType, "Timed", 50, True)
@@ -476,7 +498,7 @@ class SetupDataCollection(mforms.Box):
             self._child_controls[parent].append(checkbox)
 
         # Finally adds the control on the container
-        container.add(checkbox, False, False)
+        container.add(checkbox, False, True)
 
         return checkbox
 
@@ -496,7 +518,7 @@ class SetupDataCollection(mforms.Box):
         checkbox.add_clicked_callback(lambda: self.update_data_collection(name, checkbox))
         self.ui_fields[name] = checkbox
 
-        container.add(checkbox, False, False)
+        container.add(checkbox, False, True)
 
         return checkbox
 
@@ -574,7 +596,7 @@ class SetupDataCollection(mforms.Box):
 
         desc_label = mforms.newLabel(description)
 
-        self.add(desc_label, False, False)
+        self.add(desc_label, False, True)
 
         # Container will be the main box itself if global instrumentation
         # is unavailable, but will be changed to an inner box otherwise
@@ -588,7 +610,7 @@ class SetupDataCollection(mforms.Box):
 
             # Creates an inner box for the rest of the controls
             self.__container = mforms.newBox(False)
-            self.add(self.__container, False, False)
+            self.add(self.__container, False, True)
             self._child_controls[self.global_check] = [self.__container]
 
         # Adds the thread instrumentation checkbox 
@@ -610,7 +632,7 @@ class SetupDataCollection(mforms.Box):
         if self.owner.target_version.is_supported_mysql_version_at_least(5, 7, 3):
             self.create_section(hbox, 'transactions')
 
-        self.__container.add(hbox, False, False)
+        self.__container.add(hbox, False, True)
 
         if not self.owner.target_version.is_supported_mysql_version_at_least(5, 6, 3):
             self.create_section(hbox, 'events_waits_summary', ['by_thread_by_event_name', 'by_event_name', 'by_instance'])
@@ -622,7 +644,7 @@ class SetupDataCollection(mforms.Box):
             digest_description = "Digesting normalizes statements in a way that permits grouping similar statements and collecting information about how often they occur.\n"
             digest_label = mforms.newLabel(digest_description)
 
-            self.__container.add(digest_label, False, False)
+            self.__container.add(digest_label, False, True)
 
             self.create_dc_checkbox(self.__container, self.get_var_name_and_label('statements_digest', -1))
 
@@ -736,7 +758,7 @@ class ObjectInfoDialog(mforms.Form):
         vbox.set_padding(20)
         vbox.set_spacing(18)
 
-        caption = 'Define the database objects for which performance schema will monitor for events.\n'\
+        caption = 'Define the database objects which performance schema will monitor for events.\n'\
                   'You can use the % to indicate any schema and/or any object name.'
 
         l = mforms.newLabel(caption)
@@ -786,7 +808,7 @@ class ObjectInfoDialog(mforms.Form):
         self.timed.set_text('Timed')
         chbox.add(self.timed, True, True)
 
-        table.add(chbox, 1, 3, 3, 4, mforms.HFillFlag|mforms.HExpandFlag)
+        table.add(chbox, 1, 3, 3, 4, mforms.VFillFlag|mforms.HFillFlag|mforms.HExpandFlag)
 
         bbox = mforms.newBox(True)
         bbox.set_spacing(8)
@@ -850,11 +872,11 @@ class SetupFiltering(mforms.Box):
     def make_description_box(self, text, tooltip):
         box = mforms.newBox(True)
         box.set_spacing(12)
-        box.add(mforms.newLabel(text), False, False)
+        box.add(mforms.newLabel(text), False, True)
         l = mforms.newImageBox()
         l.set_image(mforms.App.get().get_resource_path("mini_notice.png"))
         l.set_tooltip(tooltip)
-        box.add(l, False, False)
+        box.add(l, False, True)
         return box
 
     def create_ui(self):
@@ -869,7 +891,7 @@ class SetupFiltering(mforms.Box):
         users_box.set_spacing(12)
         users_box.set_padding(12)
 
-        self.users = mforms.newTreeNodeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
+        self.users = mforms.newTreeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
         self.users.add_column(mforms.StringColumnType, "User", 150, False)
         self.users.add_column(mforms.StringColumnType, "Host", 150, False)
         self.users.end_columns()
@@ -882,20 +904,20 @@ class SetupFiltering(mforms.Box):
         description = ('Performance Schema allows defining filters to determine the connections for which data will be collected.\n'
                        'New connections having a user@host matching an entry below will be enabled for monitoring.')
         tooltip = 'Use % to indicate either any user or any host.'
-        user_buttons.add(self.make_description_box(description, tooltip), False, False)
+        user_buttons.add(self.make_description_box(description, tooltip), False, True)
 
         self.user_del_button = mforms.newButton()
         self.user_del_button.set_text('Remove')
         self.user_del_button.add_clicked_callback(self.remove_user)
         self.user_del_button.set_enabled(False)
-        user_buttons.add_end(self.user_del_button, False, False)
+        user_buttons.add_end(self.user_del_button, False, True)
 
         user_add_button = mforms.newButton()
         user_add_button.set_text('Add')
         user_add_button.add_clicked_callback(self.add_user)
-        user_buttons.add_end(user_add_button, False, False)
+        user_buttons.add_end(user_add_button, False, True)
 
-        users_box.add(user_buttons, False, False)
+        users_box.add(user_buttons, False, True)
         
         user_panel.add(users_box)
 
@@ -909,7 +931,7 @@ class SetupFiltering(mforms.Box):
         db_box.set_spacing(12)
         db_box.set_padding(12)
         
-        self.objects = mforms.newTreeNodeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
+        self.objects = mforms.newTreeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
         self.objects.add_column(mforms.StringColumnType, "Type", 100, False)
         self.objects.add_column(mforms.StringColumnType, "Schema", 200, False)
         self.objects.add_column(mforms.StringColumnType, "Object", 200, False)
@@ -931,20 +953,20 @@ class SetupFiltering(mforms.Box):
                    'The timed column indicates whether information about the events duration is recorded.')
         
         
-        db_buttons.add(self.make_description_box(description, tooltip), False, False)
+        db_buttons.add(self.make_description_box(description, tooltip), False, True)
 
         self.db_del_button = mforms.newButton()
         self.db_del_button.set_text('Remove')
         self.db_del_button.add_clicked_callback(self.remove_object)
         self.db_del_button.set_enabled(False)
-        db_buttons.add_end(self.db_del_button, False, False)
+        db_buttons.add_end(self.db_del_button, False, True)
 
         db_add_button = mforms.newButton()
         db_add_button.set_text('Add')
         db_add_button.add_clicked_callback(self.add_object)
-        db_buttons.add_end(db_add_button, False, False)
+        db_buttons.add_end(db_add_button, False, True)
 
-        db_box.add(db_buttons, False, False)
+        db_box.add(db_buttons, False, True)
 
         db_panel.add(db_box)
 
@@ -1106,9 +1128,10 @@ class SetupOptions(mforms.Box):
         return "%ss        (%s)" % (timer.name.lower().capitalize(), description)
     
     def create_timer_row(self, table, offset):
-        table.add(mforms.newLabel("%s Events" % (self._timer_names[offset].capitalize()), True), 0, 1, offset, offset + 1, mforms.HFillFlag)
+        table.add(mforms.newLabel("%s Events" % (self._timer_names[offset].capitalize()), True), 0, 1, offset, offset + 1, mforms.HFillFlag | mforms.VFillFlag)
 
         selector = mforms.newSelector(mforms.SelectorPopup)
+        selector.set_name("Timer Option Selector")
         selector.add_items([self.get_timer_type_text(timer) for timer in self._timer_types])
 
         timer_name = self._timer_names[offset]
@@ -1121,7 +1144,7 @@ class SetupOptions(mforms.Box):
 
         self._controls[timer_name] = selector
 
-        table.add(selector, 1, 2, offset, offset + 1, mforms.HFillFlag|mforms.HExpandFlag)
+        table.add(selector, 1, 2, offset, offset + 1, mforms.VFillFlag | mforms.HFillFlag | mforms.HExpandFlag)
 
         
     def create_ui(self):
@@ -1131,18 +1154,19 @@ class SetupOptions(mforms.Box):
         vbox = mforms.newBox(False)
         vbox.set_padding(12)
 
-        description = "Instruments measure the duration of events, for that they can use different timers.\n"\
+        description = "Instruments measure the duration of events, for which they can use different timers.\n"\
                       "A timer has characteristics that need to be considered when setting up the timer to be used on the\n"\
                       "different instruments:\n"\
                       "\n"\
                       "     - Frequency: Indicates the number of timer units per second.\n"\
                       "     - Resolution: Indicates the size used to increase a timer value at a time.\n"\
-                      "     -Overhead: Minimal number of cycles of overhead to obtain one timing.\n"\
+                      "     - Overhead: Minimal number of cycles of overhead to obtain one timing.\n"\
                       "\n"\
                       "Here you can configure which timer will be used for each instrument type.\n\n"
-        vbox.add(mforms.newLabel(description), False, False)
+        vbox.add(mforms.newLabel(description), False, True)
 
         table = mforms.newTable()
+        table.set_name("Timer Options Table")
         table.set_padding(1)
         table.set_row_count(5)
         table.set_column_count(2)
@@ -1152,11 +1176,11 @@ class SetupOptions(mforms.Box):
         for i in range(len(self._timer_names)):
             self.create_timer_row(table, i)
 
-        vbox.add(table, False, False)
+        vbox.add(table, False, True)
         
         panel.add(vbox)
 
-        self.add(panel, False, False)
+        self.add(panel, False, True)
 
     def refresh(self):
         for name in self._controls:
@@ -1252,7 +1276,7 @@ class SetupThreads(mforms.Box):
 
         self.add(description, False, False)
 
-        self.threads = mforms.newTreeNodeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
+        self.threads = mforms.newTreeView(mforms.TreeFlatList|mforms.TreeAltRowColors)
         self.threads.add_column(mforms.LongIntegerColumnType, "Id", 50, False)
         self.threads.add_column(mforms.StringColumnType, "Name", 250, False)
         self.threads.add_column(mforms.CheckColumnType, "Instrumented", 80, True)
@@ -1287,8 +1311,8 @@ class SetupThreads(mforms.Box):
         self._refresh_button.set_text('Refresh')
         self._refresh_button.add_clicked_callback(self.refresh_data)
 
-        bbox.add_end(self._refresh_button, False, False)
-        self.add(bbox, False, False)
+        bbox.add_end(self._refresh_button, False, True)
+        self.add(bbox, False, True)
 
     def refresh_data(self):
         # Refreshes the model class from the database
@@ -1353,29 +1377,30 @@ class SetupThreads(mforms.Box):
 
 
 class WbAdminPerformanceSchemaInstrumentation(WbAdminPSBaseTab, ChangeCounter):
-    min_server_version = (5,6,6)
+    def __init__(self, ctrl_be, instance_info, main_view):
+        WbAdminPSBaseTab.__init__(self, ctrl_be, instance_info, main_view)
+        ChangeCounter.__init__(self)
+
+        self.min_server_version = (5,6,6)
+        self.content = None
+
+        self.advanced_button = mforms.newButton()
+        self.advanced_button.add_clicked_callback(self.switch_advanced)
+        self.advanced_button.set_text("Show Advanced")
+        self.set_header(self.create_standard_header("title_instrumentation_setup.png", self.instance_info.name, "Performance Schema - Setup", self.advanced_button))
 
     @classmethod
     def wba_register(cls, admin_context):
-        admin_context.register_page(cls, "wba_performance", "Performance Schema Setup", False)
+        admin_context.register_page(cls, "Performance", "Performance Schema Setup", "Performance Schema Setup", False)
     
     @classmethod
     def identifier(cls):
         return "admin_instrumentation_setup"
 
-    def __init__(self, ctrl_be, instance_info, main_view):
-        WbAdminPSBaseTab.__init__(self, ctrl_be, instance_info, main_view)
-        ChangeCounter.__init__(self)
-
-        self.ctrl_be = ctrl_be
-        self.content = None
-
-
     @property
     def target_version(self):
         return self.ctrl_be.target_version
 
-
     def count_change(self, change, attr, value):
         ChangeCounter.count_change(self, change, attr, value)
         
@@ -1383,12 +1408,13 @@ class WbAdminPerformanceSchemaInstrumentation(WbAdminPSBaseTab, ChangeCounter):
         self.apply_button.set_enabled(self.change_count != 0)
         #self.cancel_button.set_enabled(self.change_count)
 
-    def create_ui(self):
-        self.advanced_button = mforms.newButton()
-        self.advanced_button.add_clicked_callback(self.switch_advanced)
-        self.advanced_button.set_text("Show Advanced")
-        self.create_basic_ui("title_instrumentation_setup.png", "Performance Schema - Setup", self.advanced_button)
+    def validation_failed_notification(self, failed_validation):
+        self.advanced_button.show(False)
 
+    def validation_successful_notification(self):
+        self.advanced_button.show(True)
+
+    def create_ui(self):
         self.content = mforms.newBox(False)
         self.config_tab = mforms.TabView(mforms.TabViewSystemStandard)
 
@@ -1397,10 +1423,10 @@ class WbAdminPerformanceSchemaInstrumentation(WbAdminPSBaseTab, ChangeCounter):
         self.content.add(self.config_tab, True, True)
 
         self.rebuild_ui(True)
-        self.add(self.content, True, True)
 
         self._showing_advanced = True
         self.switch_advanced()
+        return self.content
 
 
     def switch_config_tabs(self, flag):
@@ -1497,8 +1523,8 @@ class WbAdminPerformanceSchemaInstrumentation(WbAdminPSBaseTab, ChangeCounter):
             self.cancel_button.add_clicked_callback(self.on_revert_button_clicked)
             #self.cancel_button.set_enabled(False)
 
-            self.buttons.add_end(self.apply_button, False, False)
-            self.buttons.add_end(self.cancel_button, False, False)
+            self.buttons.add_end(self.apply_button, False, True)
+            self.buttons.add_end(self.cancel_button, False, True)
 
             if self.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 6):
                 self.reset_to_factory_button = mforms.newButton()
@@ -1507,7 +1533,7 @@ class WbAdminPerformanceSchemaInstrumentation(WbAdminPSBaseTab, ChangeCounter):
                 self.reset_to_factory_button.add_clicked_callback(self.on_reset_to_factory_button_clicked)
                 self.buttons.add_end(self.reset_to_factory_button, False, True)
 
-            self.content.add(self.buttons, False, False)
+            self.content.add(self.buttons, False, True)
 
         if not from_scratch and self._showing_advanced:
             self.switch_config_tabs(True)
